@@ -1,8 +1,8 @@
-/* xwrap.c - wrappers around existing library functions.
+/* xwrap.c - library function wrappers that exit instead of returning error
  *
- * Functions with the x prefix are wrappers that either succeed or kill the
- * program with an error message, but never return failure. They usually have
- * the same arguments and return value as the function they wrap.
+ * Functions with the x prefix either succeed or kill the program with an
+ * error message, so the caller doesn't have to check for failure. They
+ * usually have the same arguments and return value as the function they wrap.
  *
  * Copyright 2006 Rob Landley <rob@landley.net>
  */
@@ -54,8 +54,7 @@ void xexit(void)
 
     free(al);
   }
-  if (fflush(NULL) || ferror(stdout))
-    if (!toys.exitval) perror_msg("write");
+  if (fflush(0) || ferror(stdout)) if (!toys.exitval) perror_msg("write");
   _xexit();
 }
 
@@ -96,10 +95,8 @@ void *xrealloc(void *ptr, size_t size)
 // Die unless we can allocate a copy of this many bytes of string.
 char *xstrndup(char *s, size_t n)
 {
-  char *ret = strndup(s, ++n);
-
+  char *ret = strndup(s, n);
   if (!ret) error_exit("xstrndup");
-  ret[--n] = 0;
 
   return ret;
 }
@@ -107,7 +104,12 @@ char *xstrndup(char *s, size_t n)
 // Die unless we can allocate a copy of this string.
 char *xstrdup(char *s)
 {
-  return xstrndup(s, strlen(s));
+  long len = strlen(s);
+  char *c = xmalloc(++len);
+
+  memcpy(c, s, len);
+
+  return c;
 }
 
 void *xmemdup(void *s, long len)
@@ -129,8 +131,7 @@ char *xmprintf(char *format, ...)
   va_copy(va2, va);
 
   // How long is it?
-  len = vsnprintf(0, 0, format, va);
-  len++;
+  len = vsnprintf(0, 0, format, va)+1;
   va_end(va);
 
   // Allocate and do the sprintf()
@@ -141,6 +142,11 @@ char *xmprintf(char *format, ...)
   return ret;
 }
 
+void xferror(FILE *fp)
+{
+  if (ferror(fp)) perror_exit(fp==stdout ? "stdout" : "write");
+}
+
 void xprintf(char *format, ...)
 {
   va_list va;
@@ -148,23 +154,60 @@ void xprintf(char *format, ...)
 
   vprintf(format, va);
   va_end(va);
-  if (fflush(stdout) || ferror(stdout)) perror_exit("write");
+
+  xferror(stdout);
 }
 
+// Put string with length (does not append newline) with immediate flush
+void xputsl(char *s, int len)
+{
+  fwrite(s, 1, len, stdout);
+  fflush(stdout);
+  xferror(stdout);
+}
+
+// xputs with no newline
+void xputsn(char *s)
+{
+  xputsl(s, strlen(s));
+}
+
+// Write string to stdout with newline, checking for errors
 void xputs(char *s)
 {
-  if (EOF == puts(s) || fflush(stdout) || ferror(stdout)) perror_exit("write");
+  puts(s);
+  xferror(stdout);
 }
 
 void xputc(char c)
 {
-  if (EOF == fputc(c, stdout) || fflush(stdout) || ferror(stdout))
-    perror_exit("write");
+  fputc(c, stdout);
+  xferror(stdout);
 }
 
-void xflush(void)
+// daemonize via vfork(). Does not chdir("/"), caller should do that first
+// note: restarts process from command_main()
+void xvdaemon(void)
 {
-  if (fflush(stdout) || ferror(stdout)) perror_exit("write");;
+  int fd;
+
+  // vfork and exec /proc/self/exe
+  if (toys.stacktop) {
+    xpopen_both(0, 0);
+    _exit(0);
+  }
+
+  // new session id, point fd 0-2 at /dev/null, detach from tty
+  chdir("/");
+  setsid();
+  close(0);
+  xopen_stdio("/dev/null", O_RDWR);
+  dup2(0, 1);
+  if (-1 != (fd = open("/dev/tty", O_RDONLY))) {
+    ioctl(fd, TIOCNOTTY);
+    close(fd);
+  }
+  dup2(0, 2);
 }
 
 // This is called through the XVFORK macro because parent/child of vfork
@@ -176,7 +219,7 @@ pid_t __attribute__((returns_twice)) xvforkwrap(pid_t pid)
   if (pid == -1) perror_exit("vfork");
 
   // Signal to xexec() and friends that we vforked so can't recurse
-  toys.stacktop = 0;
+  if (!pid) toys.stacktop = 0;
 
   return pid;
 }
@@ -186,11 +229,12 @@ pid_t __attribute__((returns_twice)) xvforkwrap(pid_t pid)
 void xexec(char **argv)
 {
   // Only recurse to builtin when we have multiplexer and !vfork context.
-  if (CFG_TOYBOX && !CFG_TOYBOX_NORECURSE && toys.stacktop) toy_exec(argv);
+  if (CFG_TOYBOX && !CFG_TOYBOX_NORECURSE)
+    if (toys.stacktop && !strchr(*argv, '/')) toy_exec(argv);
   execvp(argv[0], argv);
 
+  toys.exitval = 126+(errno == ENOENT);
   perror_msg("exec %s", argv[0]);
-  toys.exitval = 127;
   if (!toys.stacktop) _exit(toys.exitval);
   xexit();
 }
@@ -198,43 +242,54 @@ void xexec(char **argv)
 // Spawn child process, capturing stdin/stdout.
 // argv[]: command to exec. If null, child re-runs original program with
 //         toys.stacktop zeroed.
-// pipes[2]: stdin, stdout of new process, only allocated if zero on way in,
-//           pass NULL to skip pipe allocation entirely.
+// pipes[2]: Filehandle to move to stdin/stdout of new process.
+//           If -1, replace with pipe handle connected to stdin/stdout.
+//           NULL treated as {0, 1}, I.E. leave stdin/stdout as is
 // return: pid of child process
-pid_t xpopen_both(char **argv, int *pipes)
+pid_t xpopen_setup(char **argv, int *pipes, void (*callback)(char **argv))
 {
   int cestnepasun[4], pid;
 
-  // Make the pipes? Note this won't set either pipe to 0 because if fds are
-  // allocated in order and if fd0 was free it would go to cestnepasun[0]
-  if (pipes) {
-    for (pid = 0; pid < 2; pid++) {
-      if (pipes[pid] != 0) continue;
-      if (pipe(cestnepasun+(2*pid))) perror_exit("pipe");
-      pipes[pid] = cestnepasun[pid+1];
-    }
-  }
+  // Make the pipes?
+  memset(cestnepasun, 0, sizeof(cestnepasun));
+  if (pipes) for (pid = 0; pid < 2; pid++)
+    if (pipes[pid]==-1 && pipe(cestnepasun+(2*pid))) perror_exit("pipe");
 
-  // Child process.
   if (!(pid = CFG_TOYBOX_FORK ? xfork() : XVFORK())) {
-    // Dance of the stdin/stdout redirection.
+    // Child process: Dance of the stdin/stdout redirection.
+    // cestnepasun[1]->cestnepasun[0] and cestnepasun[3]->cestnepasun[2]
     if (pipes) {
       // if we had no stdin/out, pipe handles could overlap, so test for it
       // and free up potentially overlapping pipe handles before reuse
-      if (pipes[1] != -1) close(cestnepasun[2]);
-      if (pipes[0] != -1) {
-        close(cestnepasun[1]);
-        if (cestnepasun[0]) {
-          dup2(cestnepasun[0], 0);
-          close(cestnepasun[0]);
-        }
+
+      // in child, close read end of output pipe, use write end as new stdout
+      if (cestnepasun[2]) {
+        close(cestnepasun[2]);
+        pipes[1] = cestnepasun[3];
       }
-      if (pipes[1] != -1) {
-        dup2(cestnepasun[3], 1);
-        dup2(cestnepasun[3], 2);
-        if (cestnepasun[3] > 2 || !cestnepasun[3]) close(cestnepasun[3]);
+
+      // in child, close write end of input pipe, use read end as new stdin
+      if (cestnepasun[1]) {
+        close(cestnepasun[1]);
+        pipes[0] = cestnepasun[0];
+      }
+
+      // If swapping stdin/stdout, dup a filehandle that gets closed before use
+      if (!pipes[1]) pipes[1] = dup(0);
+
+      // Are we redirecting stdin?
+      if (pipes[0]) {
+        dup2(pipes[0], 0);
+        close(pipes[0]);
+      }
+
+      // Are we redirecting stdout?
+      if (pipes[1] != 1) {
+        dup2(pipes[1], 1);
+        close(pipes[1]);
       }
     }
+    if (callback) callback(argv);
     if (argv) xexec(argv);
 
     // In fork() case, force recursion because we know it's us.
@@ -252,37 +307,51 @@ pid_t xpopen_both(char **argv, int *pipes)
       // setting high bit of argv[0][0] to let new process know
       **toys.argv |= 0x80;
       execv(s, toys.argv);
+      if ((s = getenv("_"))) execv(s, toys.argv);
       perror_msg_raw(s);
 
       _exit(127);
     }
   }
 
-  // Parent process
+  // Parent process: vfork had a shared environment, clean up.
   if (!CFG_TOYBOX_FORK) **toys.argv &= 0x7f;
+
   if (pipes) {
-    if (pipes[0] != -1) close(cestnepasun[0]);
-    if (pipes[1] != -1) close(cestnepasun[3]);
+    if (cestnepasun[1]) {
+      pipes[0] = cestnepasun[1];
+      close(cestnepasun[0]);
+    }
+    if (cestnepasun[2]) {
+      pipes[1] = cestnepasun[2];
+      close(cestnepasun[3]);
+    }
   }
 
   return pid;
 }
 
+pid_t xpopen_both(char **argv, int *pipes)
+{
+  return xpopen_setup(argv, pipes, 0);
+}
+
+
 // Wait for child process to exit, then return adjusted exit code.
 int xwaitpid(pid_t pid)
 {
-  int status;
+  int status = 127<<8;
 
-  while (-1 == waitpid(pid, &status, 0) && errno == EINTR);
+  while (-1 == waitpid(pid, &status, 0) && errno == EINTR) errno = 0;
 
-  return WIFEXITED(status) ? WEXITSTATUS(status) : WTERMSIG(status)+127;
+  return WIFEXITED(status) ? WEXITSTATUS(status) : WTERMSIG(status)+128;
 }
 
 int xpclose_both(pid_t pid, int *pipes)
 {
   if (pipes) {
-    close(pipes[0]);
-    close(pipes[1]);
+    if (pipes[0]) close(pipes[0]);
+    if (pipes[1]>1) close(pipes[1]);
   }
 
   return xwaitpid(pid);
@@ -293,8 +362,8 @@ pid_t xpopen(char **argv, int *pipe, int isstdout)
 {
   int pipes[2], pid;
 
-  pipes[!isstdout] = -1;
-  pipes[!!isstdout] = 0;
+  pipes[0] = isstdout ? 0 : -1;
+  pipes[1] = isstdout ? -1 : 1;
   pid = xpopen_both(argv, pipes);
   *pipe = pid ? pipes[!!isstdout] : -1;
 
@@ -314,6 +383,34 @@ int xrun(char **argv)
   return xpclose_both(xpopen_both(argv, 0), 0);
 }
 
+// Run child, writing to_stdin, returning stdout or NULL, pass through stderr
+char *xrunread(char *argv[], char *to_stdin)
+{
+  char *result = 0;
+  int pipe[] = {-1, -1}, total = 0, len;
+  pid_t pid;
+
+  pid = xpopen_both(argv, pipe);
+  if (to_stdin && *to_stdin) writeall(*pipe, to_stdin, strlen(to_stdin));
+  close(*pipe);
+  for (;;) {
+    if (0>=(len = readall(pipe[1], libbuf, sizeof(libbuf)))) break;
+    memcpy((result = xrealloc(result, 1+total+len))+total, libbuf, len);
+    total += len;
+    if (len != sizeof(libbuf)) break;
+  }
+  if (result) result[total] = 0;
+  close(pipe[1]);
+
+  if (xwaitpid(pid)) {
+    free(result);
+
+    return 0;
+  }
+
+  return result;
+}
+
 void xaccess(char *path, int flags)
 {
   if (access(path, flags)) perror_exit("Can't access '%s'", path);
@@ -332,7 +429,7 @@ int xcreate_stdio(char *path, int flags, int mode)
 {
   int fd = open(path, (flags^O_CLOEXEC)&~WARN_ONLY, mode);
 
-  if (fd == -1) ((mode&WARN_ONLY) ? perror_msg_raw : perror_exit_raw)(path);
+  if (fd == -1) ((flags&WARN_ONLY) ? perror_msg_raw : perror_exit_raw)(path);
   return fd;
 }
 
@@ -349,7 +446,7 @@ void xpipe(int *pp)
 
 void xclose(int fd)
 {
-  if (close(fd)) perror_exit("xclose");
+  if (fd != -1 && close(fd)) perror_exit("xclose");
 }
 
 int xdup(int fd)
@@ -361,9 +458,7 @@ int xdup(int fd)
   return fd;
 }
 
-// Move file descriptor above stdin/stdout/stderr, using /dev/null to consume
-// old one. (We should never be called with stdin/stdout/stderr closed, but...)
-int notstdio(int fd)
+int xnotstdio(int fd)
 {
   if (fd<0) return fd;
 
@@ -378,11 +473,16 @@ int notstdio(int fd)
   return fd;
 }
 
+void xrename(char *from, char *to)
+{
+  if (rename(from, to)) perror_exit("rename %s -> %s", from, to);
+}
+
 int xtempfile(char *name, char **tempname)
 {
   int fd;
 
-   *tempname = xmprintf("%s%s", name, "XXXXXX");
+  *tempname = xmprintf("%s%s", name, "XXXXXX");
   if(-1 == (fd = mkstemp(*tempname))) error_exit("no temp file");
 
   return fd;
@@ -391,13 +491,13 @@ int xtempfile(char *name, char **tempname)
 // Create a file but don't return stdin/stdout/stderr
 int xcreate(char *path, int flags, int mode)
 {
-  return notstdio(xcreate_stdio(path, flags, mode));
+  return xnotstdio(xcreate_stdio(path, flags, mode));
 }
 
 // Open a file descriptor NOT in stdin/stdout/stderr
 int xopen(char *path, int flags)
 {
-  return notstdio(xopen_stdio(path, flags));
+  return xnotstdio(xopen_stdio(path, flags));
 }
 
 // Open read only, treating "-" as a synonym for stdin, defaulting to warn only
@@ -477,72 +577,74 @@ void xstat(char *path, struct stat *st)
   if(stat(path, st)) perror_exit("Can't stat %s", path);
 }
 
-// Cannonicalize path, even to file with one or more missing components at end.
-// if exact, require last path component to exist
-char *xabspath(char *path, int exact)
+// Canonicalize path, even to file with one or more missing components at end.
+// Returns allocated string for pathname or NULL if doesn't exist. Flags are:
+// ABS_PATH:path to last component must exist ABS_FILE: whole path must exist
+// ABS_KEEP:keep symlinks in path ABS_LAST: keep symlink at end of path
+char *xabspath(char *path, int flags)
 {
-  struct string_list *todo, *done = 0;
-  int try = 9999, dirfd = open("/", 0), missing = 0;
-  char *ret;
+  struct string_list *todo, *done = 0, *new, **tail;
+  int fd, track, len, try = 9999, dirfd = -1, missing = 0;
+  char *str;
 
-  // If this isn't an absolute path, start with cwd.
+  // If the last file must exist, path to it must exist.
+  if (flags&ABS_FILE) flags |= ABS_PATH;
+  // If we don't resolve path's symlinks, don't resolve last symlink.
+  if (flags&ABS_KEEP) flags |= ABS_LAST;
+
+  // If this isn't an absolute path, start with cwd or $PWD.
   if (*path != '/') {
-    char *temp = xgetcwd();
-
-    splitpath(path, splitpath(temp, &todo));
-    free(temp);
+    if ((flags & ABS_KEEP) && (str = getenv("PWD")))
+      splitpath(path, splitpath(str, &todo));
+    else {
+      splitpath(path, splitpath(str = xgetcwd(), &todo));
+      free(str);
+    }
   } else splitpath(path, &todo);
 
   // Iterate through path components in todo, prepend processed ones to done.
   while (todo) {
-    struct string_list *new = llist_pop(&todo), **tail;
-    ssize_t len;
-
-    // Eventually break out of endless loops
+    // break out of endless symlink loops
     if (!try--) {
       errno = ELOOP;
       goto error;
     }
 
-    // Removable path componenents.
-    if (!strcmp(new->str, ".") || !strcmp(new->str, "..")) {
-      int x = new->str[1];
-
+    // Remove . or .. component, tracking dirfd back up tree as necessary
+    str = (new = llist_pop(&todo))->str;
+    // track dirfd if this component must exist or we're resolving symlinks
+    track = ((flags>>!todo) & (ABS_PATH|ABS_KEEP)) ^ ABS_KEEP;
+    if (!done && track) dirfd = open("/", O_PATH);
+    if (*str=='.' && !str[1+((fd = str[1])=='.')]) {
       free(new);
-      if (!x) continue;
-      if (done) free(llist_pop(&done));
-      len = 0;
-
-      if (missing) missing--;
-      else {
-        if (-1 == (x = openat(dirfd, "..", 0))) goto error;
-        close(dirfd);
-        dirfd = x;
+      if (fd) {
+        if (done) free(llist_pop(&done));
+        if (missing) missing--;
+        else if (track) {
+          if (-1 == (fd = openat(dirfd, "..", O_PATH))) goto error;
+          close(dirfd);
+          dirfd = fd;
+        }
       }
       continue;
     }
 
     // Is this a symlink?
-    len = readlinkat(dirfd, new->str, libbuf, sizeof(libbuf));
+    if (flags & (ABS_KEEP<<!todo)) len = 0, errno = EINVAL;
+    else len = readlinkat(dirfd, str, libbuf, sizeof(libbuf));
     if (len>4095) goto error;
 
     // Not a symlink: add to linked list, move dirfd, fail if error
     if (len<1) {
-      int fd;
-
       new->next = done;
       done = new;
-      if (errno == EINVAL && !todo) break;
-      if (errno == ENOENT && exact<0) {
-        missing++;
-        continue;
+      if (errno == ENOENT && !(flags & (ABS_PATH<<!todo))) missing++;
+      else if (errno != EINVAL && (flags & (ABS_PATH<<!todo))) goto error;
+      else if (track) {
+        if (-1 == (fd = openat(dirfd, new->str, O_PATH))) goto error;
+        close(dirfd);
+        dirfd = fd;
       }
-      if (errno != EINVAL && (exact || todo)) goto error;
-
-      fd = openat(dirfd, new->str, 0);
-      if (fd == -1 && (exact || todo || errno != ENOENT)) goto error;
-      close(dirfd);
-      dirfd = fd;
       continue;
     }
 
@@ -550,13 +652,13 @@ char *xabspath(char *path, int exact)
     libbuf[len] = 0;
     if (*libbuf == '/') {
       llist_traverse(done, free);
-      done=0;
+      done = 0;
       close(dirfd);
-      dirfd = open("/", 0);
+      dirfd = -1;
     }
     free(new);
 
-    // prepend components of new path. Note symlink to "/" will leave new NULL
+    // prepend components of new path. Note symlink to "/" will leave new = NULL
     tail = splitpath(libbuf, &new);
 
     // symlink to "/" will return null and leave tail alone
@@ -565,14 +667,13 @@ char *xabspath(char *path, int exact)
       todo = new;
     }
   }
-  close(dirfd);
-
-  // At this point done has the path, in reverse order. Reverse list while
-  // calculating buffer length.
+  xclose(dirfd);
 
+  // At this point done has the path, in reverse order. Reverse list
+  // (into todo) while calculating buffer length.
   try = 2;
   while (done) {
-    struct string_list *temp = llist_pop(&done);;
+    struct string_list *temp = llist_pop(&done);
 
     if (todo) try++;
     try += strlen(temp->str);
@@ -581,20 +682,18 @@ char *xabspath(char *path, int exact)
   }
 
   // Assemble return buffer
-
-  ret = xmalloc(try);
-  *ret = '/';
-  ret [try = 1] = 0;
+  *(str = xmalloc(try)) = '/';
+  str[try = 1] = 0;
   while (todo) {
-    if (try>1) ret[try++] = '/';
-    try = stpcpy(ret+try, todo->str) - ret;
+    if (try>1) str[try++] = '/';
+    try = stpcpy(str+try, todo->str) - str;
     free(llist_pop(&todo));
   }
 
-  return ret;
+  return str;
 
 error:
-  close(dirfd);
+  xclose(dirfd);
   llist_traverse(todo, free);
   llist_traverse(done, free);
 
@@ -603,7 +702,7 @@ error:
 
 void xchdir(char *path)
 {
-  if (chdir(path)) error_exit("chdir '%s'", path);
+  if (chdir(path)) perror_exit("chdir '%s'", path);
 }
 
 void xchroot(char *path)
@@ -682,7 +781,7 @@ void xsetuser(struct passwd *pwd)
 
 // This can return null (meaning file not found).  It just won't return null
 // for memory allocation reasons.
-char *xreadlink(char *name)
+char *xreadlinkat(int dir, char *name)
 {
   int len, size = 0;
   char *buf = 0;
@@ -691,7 +790,7 @@ char *xreadlink(char *name)
   for(;;) {
     size +=64;
     buf = xrealloc(buf, size);
-    len = readlink(name, buf, size);
+    len = readlinkat(dir, name, buf, size);
 
     if (len<0) {
       free(buf);
@@ -704,6 +803,12 @@ char *xreadlink(char *name)
   }
 }
 
+char *xreadlink(char *name)
+{
+  return xreadlinkat(AT_FDCWD, name);
+}
+
+
 char *xreadfile(char *name, char *buf, off_t len)
 {
   if (!(buf = readfile(name, buf, len))) perror_exit("Bad '%s'", name);
@@ -757,22 +862,39 @@ void xpidfile(char *name)
   close(fd);
 }
 
-// Copy the rest of in to out and close both files.
-
-long long xsendfile(int in, int out)
+// error_exit if we couldn't copy all bytes
+long long xsendfile_len(int in, int out, long long bytes)
 {
-  long long total = 0;
-  long len;
+  long long len = sendfile_len(in, out, bytes, 0);
 
-  if (in<0) return 0;
-  for (;;) {
-    len = xread(in, libbuf, sizeof(libbuf));
-    if (len<1) break;
-    xwrite(out, libbuf, len);
-    total += len;
+  if (bytes != -1 && bytes != len) {
+    if (out == 1 && len<0) xexit();
+    error_exit("short %s", (len<0) ? "write" : "read");
   }
 
-  return total;
+  return len;
+}
+
+// warn and pad with zeroes if we couldn't copy all bytes
+void xsendfile_pad(int in, int out, long long len)
+{
+  len -= xsendfile_len(in, out, len);
+  if (len) {
+    perror_msg("short read");
+    memset(libbuf, 0, sizeof(libbuf));
+    while (len) {
+      int i = len>sizeof(libbuf) ? sizeof(libbuf) : len;
+
+      xwrite(out, libbuf, i);
+      len -= i;
+    }
+  }
+}
+
+// copy all of in to out
+long long xsendfile(int in, int out)
+{
+  return xsendfile_len(in, out, -1);
 }
 
 double xstrtod(char *s)
@@ -830,16 +952,27 @@ long long xparsemillitime(char *arg)
   return (l*1000LL)+ll;
 }
 
+void xparsetimespec(char *arg, struct timespec *ts)
+{
+  ts->tv_sec = xparsetime(arg, 9, &ts->tv_nsec);
+}
 
 
 // Compile a regular expression into a regex_t
 void xregcomp(regex_t *preg, char *regex, int cflags)
 {
-  int rc = regcomp(preg, regex, cflags);
+  int rc;
 
-  if (rc) {
+  // BSD regex implementations don't support the empty regex (which isn't
+  // allowed in the POSIX grammar), but glibc does. Fake it for BSD.
+  if (!*regex) {
+    regex = "()";
+    cflags |= REG_EXTENDED;
+  }
+
+  if ((rc = regcomp(preg, regex, cflags))) {
     regerror(rc, preg, libbuf, sizeof(libbuf));
-    error_exit("xregcomp: %s", libbuf);
+    error_exit("bad regex '%s': %s", regex, libbuf);
   }
 }
 
@@ -870,3 +1003,143 @@ void xsignal(int signal, void *handler)
 {
   xsignal_flags(signal, handler, 0);
 }
+
+
+time_t xvali_date(struct tm *tm, char *str)
+{
+  time_t t;
+
+  if (tm && (unsigned)tm->tm_sec<=60 && (unsigned)tm->tm_min<=59
+     && (unsigned)tm->tm_hour<=23 && tm->tm_mday && (unsigned)tm->tm_mday<=31
+     && (unsigned)tm->tm_mon<=11 && (t = mktime(tm)) != -1) return t;
+
+  error_exit("bad date %s", str);
+}
+
+// Parse date string (relative to current *t). Sets time_t and nanoseconds.
+void xparsedate(char *str, time_t *t, unsigned *nano, int endian)
+{
+  struct tm tm;
+  time_t now = *t;
+  int len = 0, i = 0;
+  long long ll;
+  // Formats with seconds come first. Posix can't agree on whether 12 digits
+  // has year before (touch -t) or year after (date), so support both.
+  char *s = str, *p, *oldtz = 0, *formats[] = {"%Y-%m-%d %T", "%Y-%m-%dT%T",
+    "%a %b %e %H:%M:%S %Z %Y", // date(1) output format in POSIX/C locale.
+    "%H:%M:%S", "%Y-%m-%d %H:%M", "%Y-%m-%d", "%H:%M", "%m%d%H%M",
+    endian ? "%m%d%H%M%y" : "%y%m%d%H%M",
+    endian ? "%m%d%H%M%C%y" : "%C%y%m%d%H%M"};
+
+  *nano = 0;
+
+  // Parse @UNIXTIME[.FRACTION]
+  if (1 == sscanf(s, "@%lld%n", &ll, &len)) {
+    if (*(s+=len)=='.') for (len = 0, s++; len<9; len++) {
+      *nano *= 10;
+      if (isdigit(*s)) *nano += *s++-'0';
+    }
+    // Can't be sure t is 64 bit (yet) for %lld above
+    *t = ll;
+    if (!*s) return;
+    xvali_date(0, str);
+  }
+
+  // Try each format
+  for (i = 0; i<ARRAY_LEN(formats); i++) {
+    localtime_r(&now, &tm);
+    tm.tm_hour = tm.tm_min = tm.tm_sec = 0;
+    tm.tm_isdst = -endian;
+
+    if ((p = strptime(s, formats[i], &tm))) {
+      // Handle optional fractional seconds.
+      if (*p == '.') {
+        p++;
+        // If format didn't already specify seconds, grab seconds
+        if (i>2) {
+          len = 0;
+          sscanf(p, "%2u%n", &tm.tm_sec, &len);
+          p += len;
+        }
+        // nanoseconds
+        for (len = 0; len<9; len++) {
+          *nano *= 10;
+          if (isdigit(*p)) *nano += *p++-'0';
+        }
+      }
+
+      // Handle optional Z or +HH[[:]MM] timezone
+      while (isspace(*p)) p++;
+      if (*p && strchr("Z+-", *p)) {
+        unsigned uu[3] = {0}, n = 0, nn = 0;
+        char *tz = 0, sign = *p++;
+
+        if (sign == 'Z') tz = "UTC0";
+        else if (0<sscanf(p, " %u%n : %u%n : %u%n", uu,&n,uu+1,&nn,uu+2,&nn)) {
+          if (n>2) {
+            uu[1] += uu[0]%100;
+            uu[0] /= 100;
+          }
+          if (n>nn) nn = n;
+          if (!nn) continue;
+
+          // flip sign because POSIX UTC offsets are backwards
+          sprintf(tz = libbuf, "UTC%c%02u:%02u:%02u", "+-"[sign=='+'],
+            uu[0], uu[1], uu[2]);
+          p += nn;
+        }
+
+        if (!oldtz) {
+          oldtz = getenv("TZ");
+          if (oldtz) oldtz = xstrdup(oldtz);
+        }
+        if (tz) setenv("TZ", tz, 1);
+      }
+      while (isspace(*p)) p++;
+
+      if (!*p) break;
+    }
+  }
+
+  // Sanity check field ranges
+  *t = xvali_date((i!=ARRAY_LEN(formats)) ? &tm : 0, str);
+
+  if (oldtz) setenv("TZ", oldtz, 1);
+  free(oldtz);
+}
+
+// Return line of text from file.
+char *xgetdelim(FILE *fp, int delim)
+{
+  char *new = 0;
+  size_t len = 0;
+  long ll;
+
+  errno = 0;
+  if (1>(ll = getdelim(&new, &len, delim, fp))) {
+    if (errno && errno != EINTR) perror_msg("getline");
+    free(new);
+    new = 0;
+  }
+
+  return new;
+}
+
+// Return line of text from file. Strips trailing newline (if any).
+char *xgetline(FILE *fp)
+{
+  return chomp(xgetdelim(fp, '\n'));
+}
+
+time_t xmktime(struct tm *tm, int utc)
+{
+  char *old_tz = utc ? xtzset("UTC0") : 0;
+  time_t result;
+
+  if ((result = mktime(tm)) < 0) error_exit("mktime");
+  if (utc) {
+    free(xtzset(old_tz));
+    free(old_tz);
+  }
+  return result;
+}
