@@ -42,6 +42,7 @@ config IFCONFIG
     broadcast ADDR   - Set broadcast address
     pointopoint ADDR - PPP and PPPOE use this instead of "route add default gw"
     hw TYPE ADDR     - set hardware (mac) address (type = ether|infiniband)
+    rename NEWNAME   - rename interface
 
     Flags you can set on an interface (or -remove by prefixing with -):
 
@@ -120,7 +121,7 @@ static void display_ifconfig(char *name, int always, unsigned long long val[])
   flags = ifre.ifr_flags;
   if (!always && !(flags & IFF_UP)) return;
 
-  if (toys.optflags&FLAG_S) {
+  if (FLAG(S)) {
     unsigned uu = 0;
     int len;
 
@@ -133,10 +134,11 @@ static void display_ifconfig(char *name, int always, unsigned long long val[])
     printf("%*c", 29-len, ' ');
   }
 
-  // query hardware type and hardware address
-  xioctl(TT.sockfd, SIOCGIFHWADDR, &ifre);
+  // Query hardware type and hardware address.
+  // Not xioctl because you don't have permission for this on Android.
+  ioctl(TT.sockfd, SIOCGIFHWADDR, &ifre);
 
-  if (toys.optflags&FLAG_S)
+  if (FLAG(S))
     for (i=0; i<6; i++) printf(":%02x"+!i, ifre.ifr_hwaddr.sa_data[i]);
   else {
     for (i=0; i < ARRAY_LEN(types)-1; i++)
@@ -160,7 +162,7 @@ static void display_ifconfig(char *name, int always, unsigned long long val[])
   pp = (char *)&ifre.ifr_addr;
   for (i = 0; i<sizeof(ifre.ifr_addr); i++) if (pp[i]) break;
 
-  if (!(toys.optflags&FLAG_S) && i != sizeof(ifre.ifr_addr)) {
+  if (!FLAG(S) && i != sizeof(ifre.ifr_addr)) {
     struct sockaddr_in *si = (struct sockaddr_in *)&ifre.ifr_addr;
     struct {
       char *name;
@@ -208,7 +210,7 @@ static void display_ifconfig(char *name, int always, unsigned long long val[])
         char *ptr = ipv6_addr+sizeof(ipv6_addr)-1;
 
         // convert giant hex string into colon-spearated ipv6 address by
-        // inserting ':' every 4 characters. 
+        // inserting ':' every 4 characters.
         for (i = 32; i; i--)
           if ((*(ptr--) = ipv6_addr[i])) if (!(i&3)) *(ptr--) = ':';
 
@@ -220,7 +222,7 @@ static void display_ifconfig(char *name, int always, unsigned long long val[])
 
             for (i=0; i<ARRAY_LEN(scopes); i++)
               if (iscope == (!!i)<<(i+3)) scope = scopes[i];
-            if (toys.optflags&FLAG_S) xprintf(" %s/%d@%c", toybuf, plen,*scope);
+            if (FLAG(S)) xprintf(" %s/%d@%c", toybuf, plen,*scope);
             else xprintf("%10cinet6 addr: %s/%d Scope: %s\n",
                          ' ', toybuf, plen, scope);
           }
@@ -230,7 +232,7 @@ static void display_ifconfig(char *name, int always, unsigned long long val[])
     fclose(fp);
   }
 
-  if (toys.optflags&FLAG_S) {
+  if (FLAG(S)) {
     xputc('\n');
     return;
   }
@@ -242,12 +244,12 @@ static void display_ifconfig(char *name, int always, unsigned long long val[])
     char **s, *str[] = {
       "UP", "BROADCAST", "DEBUG", "LOOPBACK", "POINTOPOINT", "NOTRAILERS",
       "RUNNING", "NOARP", "PROMISC", "ALLMULTI", "MASTER", "SLAVE", "MULTICAST",
-      "PORTSEL", "AUTOMEDIA", "DYNAMIC", NULL
+      "PORTSEL", "AUTOMEDIA", "DYNAMIC", "LOWER_UP", "DORMANT", "ECHO", NULL
     };
 
     for (s = str; *s; s++) {
       if (flags & mask) xprintf("%s ", *s);
-      mask = mask << 1;
+      mask <<= 1;
     }
   } else xprintf("[NO FLAGS] ");
 
@@ -330,7 +332,7 @@ static void show_iface(char *iface_name)
       sl->next = ifaces;
       ifaces = sl;
 
-      display_ifconfig(sl->str, toys.optflags & FLAG_a, val);
+      display_ifconfig(sl->str, FLAG(a), val);
     }
   }
   fclose(fp);
@@ -357,7 +359,7 @@ static void show_iface(char *iface_name)
       for(sl = ifaces; sl; sl = sl->next)
         if(!strcmp(sl->str, ifre->ifr_name)) break;
 
-      if(!sl) display_ifconfig(ifre->ifr_name, toys.optflags & FLAG_a, 0);
+      if(!sl) display_ifconfig(ifre->ifr_name, FLAG(a), 0);
     }
 
     free(ifcon.ifc_buf);
@@ -404,6 +406,7 @@ void ifconfig_main(void)
       {"netmask", 0, SIOCSIFNETMASK},
       {"dstaddr", 0, SIOCSIFDSTADDR},
       {"mtu", IFREQ_OFFSZ(ifr_mtu), SIOCSIFMTU},
+      {"rename", IFREQ_OFFSZ(ifr_newname), SIOCSIFNAME},
       {"keepalive", IFREQ_OFFSZ(ifr_data), SIOCDEVPRIVATE}, // SIOCSKEEPALIVE
       {"outfill", IFREQ_OFFSZ(ifr_data), SIOCDEVPRIVATE+2}, // SIOCSOUTFILL
       {"metric", IFREQ_OFFSZ(ifr_metric), SIOCSIFMETRIC},
@@ -444,7 +447,7 @@ void ifconfig_main(void)
 
         if (*hw_addr == ':') hw_addr++;
         sscanf(hw_addr, "%2x%n", &val, &len);
-        if (!len || len > 2) break; // 1 nibble can be set e.g. C2:79:38:95:D:A 
+        if (!len || len > 2) break; // 1 nibble can be set e.g. C2:79:38:95:D:A
         hw_addr += len;
         *p++ = val;
       }
@@ -492,6 +495,7 @@ void ifconfig_main(void)
       struct argh *t = try+i;
       int on = t->on, off = t->off;
 
+      // First entry in list assigns address to interface (no command name)
       if (!t->name) {
         if (isdigit(**argv) || !strcmp(*argv, "default")) argv--;
         else continue;
@@ -504,11 +508,12 @@ void ifconfig_main(void)
 
           // Assign value to ifre field and call ioctl? (via IFREQ_OFFSZ.)
           if (on < 0) {
-            long l = strtoul(*argv, 0, 0);
+            void *dest = ((on = -on)>>16)+(char *)&ifre;
 
+            // If we're about to set mem_start/io_addr/irq, get other 2 first
             if (off == SIOCSIFMAP) xioctl(TT.sockfd, SIOCGIFMAP, &ifre);
-            on = -on;
-            poke((on>>16) + (char *)&ifre, l, on&15);
+            if (off == SIOCSIFNAME) xstrncpy(dest, *argv, on&0xffff);
+            else poke(dest, strtoul(*argv, 0, 0), on&15);
             xioctl(TT.sockfd, off, &ifre);
             break;
           } else {
