@@ -3,7 +3,8 @@
  * Copyright 2012 Madhur Verma <mad.flexi@gmail.com>
  * Copyright 2013 Kyungwan Han <asura321@gmail.com>
  *
- * Not in SUSv4.
+ * See RFC 2132, and 3442 (option 121)
+
 USE_DHCP(NEWTOY(dhcp, "V:H:F:x*r:O*A#<0=20T#<0=3t#<0=3s:p:i:SBRCaovqnbf", TOYFLAG_SBIN|TOYFLAG_ROOTONLY))
 
 config DHCP
@@ -13,7 +14,7 @@ config DHCP
    usage: dhcp [-fbnqvoCRB] [-i IFACE] [-r IP] [-s PROG] [-p PIDFILE]
                [-H HOSTNAME] [-V VENDOR] [-x OPT:VAL] [-O OPT]
 
-        Configure network dynamicaly using DHCP.
+        Configure network dynamically using DHCP.
 
       -i Interface to use (default eth0)
       -p Create pidfile
@@ -134,7 +135,7 @@ GLOBALS(
 
 static void (*dbg)(char *format, ...);
 static void dummy(char *format, ...){
-	return;
+  return;
 }
 
 typedef struct dhcpc_result_s {
@@ -247,7 +248,7 @@ static  struct sock_filter filter_instr[] = {
 };
 
 static  struct sock_fprog filter_prog = {
-    .len = ARRAY_LEN(filter_instr), 
+    .len = ARRAY_LEN(filter_instr),
     .filter = (struct sock_filter *) filter_instr,
 };
 
@@ -428,7 +429,7 @@ static int strtoopt( char *str, uint8_t optonly)
     options_list[count].len = sizeof(uint32_t);
     options_list[count].val = xmalloc(sizeof(uint32_t));
     convtmp = strtou32(valstr);
-    if (convtmp < 0) error_exit("Invalid/wrong formated number %s", valstr);
+    if (convtmp < 0) error_exit("Invalid/wrong formatted number %s", valstr);
     convtmp = htonl(convtmp);
     memcpy(options_list[count].val, &convtmp, sizeof(uint32_t));
     break;
@@ -464,7 +465,7 @@ static int strtoopt( char *str, uint8_t optonly)
       valstr = strtok(NULL," \t");
     }
     break;
-  case DHCP_STRLST: 
+  case DHCP_STRLST:
   case DHCP_IPPLST:
     break;
   case DHCP_STCRTS:
@@ -650,7 +651,6 @@ static int mode_app(void)
 static int read_raw(void)
 {
   dhcp_raw_t packet;
-  uint16_t check;
   int bytes = 0;
 
   memset(&packet, 0, sizeof(packet));
@@ -668,7 +668,7 @@ static int read_raw(void)
   }
   // ignore any extra garbage bytes
   bytes = ntohs(packet.iph.tot_len);
-  // make sure its the right packet for us, and that it passes sanity checks 
+  // make sure its the right packet for us, and that it passes sanity checks
   if (packet.iph.protocol != IPPROTO_UDP || packet.iph.version != IPVERSION
    || packet.iph.ihl != (sizeof(packet.iph) >> 2)
    || packet.udph.dest != htons(DHCPC_CLIENT_PORT)
@@ -676,18 +676,19 @@ static int read_raw(void)
     dbg("\tUnrelated/bogus packet, ignoring\n");
     return -2;
   }
-  // verify IP checksum
-  check = packet.iph.check;
-  packet.iph.check = 0;
-  if (check != dhcp_checksum(&packet.iph, sizeof(packet.iph))) {
+  // Verify IP checksum.
+  if (dhcp_checksum(&packet.iph, sizeof(packet.iph)) != 0) {
     dbg("\tBad IP header checksum, ignoring\n");
     return -2;
   }
+  // Verify UDP checksum. From RFC 768, the UDP checksum is done over the IPv4
+  // pseudo header, the UDP header and the UDP data. The IPv4 pseudo header
+  // includes saddr, daddr, protocol, and UDP length. The IP header has to be
+  // modified for this.
   memset(&packet.iph, 0, ((size_t) &((struct iphdr *)0)->protocol));
+  packet.iph.check = 0;
   packet.iph.tot_len = packet.udph.len;
-  check = packet.udph.check;
-  packet.udph.check = 0;
-  if (check && check != dhcp_checksum(&packet, bytes)) {
+  if (packet.udph.check != 0 && dhcp_checksum(&packet, bytes) != 0) {
     dbg("\tPacket with bad UDP checksum received, ignoring\n");
     return -2;
   }
@@ -1057,9 +1058,7 @@ static uint8_t dhcpc_parseoptions(dhcpc_result_t *presult, uint8_t *optptr)
 {
   uint8_t type = 0, *options, overloaded = 0;;
   uint16_t flag = 0;
-  uint32_t convtmp = 0;
   char *dest, *pfx;
-  struct in_addr addr;
   int count, optlen, size = ARRAY_LEN(options_list);
 
   if (toys.optflags & FLAG_x) {
@@ -1102,51 +1101,35 @@ static uint8_t dhcpc_parseoptions(dhcpc_result_t *presult, uint8_t *optptr)
         break;
       }
     }
+    msgopt_list[count].val = 0;
+    msgopt_list[count].len = 0;
     switch (flag) {
     case DHCP_NUM32:
-      memcpy(&convtmp, &optptr[2], sizeof(uint32_t));
-      convtmp = htonl(convtmp);
-      sprintf(toybuf, "%u", convtmp);
-      msgopt_list[count].val = strdup(toybuf);
-      msgopt_list[count].len = strlen(toybuf);
+      msgopt_list[count].val = xmprintf("%llu", peek_be(optptr+2, 4));
       break;
     case DHCP_NUM16:
-      memcpy(&convtmp, &optptr[2], sizeof(uint16_t));
-      convtmp = htons(convtmp);
-      sprintf(toybuf, "%u", convtmp);
-      msgopt_list[count].val = strdup(toybuf);
-      msgopt_list[count].len = strlen(toybuf);
+      msgopt_list[count].val = xmprintf("%llu", peek_be(optptr+2, 2));
       break;
     case DHCP_NUM8:
-      memcpy(&convtmp, &optptr[2], sizeof(uint8_t));
-      sprintf(toybuf, "%u", convtmp);
-      msgopt_list[count].val = strdup(toybuf);
-      msgopt_list[count].len = strlen(toybuf);
+      msgopt_list[count].val = xmprintf("%llu", peek_be(optptr+2, 1));
       break;
     case DHCP_IP:
-      memcpy(&convtmp, &optptr[2], sizeof(uint32_t));
-      addr.s_addr = convtmp;
-      sprintf(toybuf, "%s", inet_ntoa(addr));
-      msgopt_list[count].val = strdup(toybuf);
-      msgopt_list[count].len = strlen(toybuf);
+      msgopt_list[count].val = xstrdup(inet_ntoa((struct in_addr){peek(optptr+2, 4)}));
       break;
     case DHCP_STRING:
-      sprintf(toybuf, "%.*s", optptr[1], &optptr[2]);
-      msgopt_list[count].val = strdup(toybuf);
-      msgopt_list[count].len = strlen(toybuf);
+      msgopt_list[count].val = xmprintf("%.*s", optptr[1], optptr+2);
       break;
     case DHCP_IPLIST:
+      options = &optptr[2];
       optlen = optptr[1];
       dest = toybuf;
       while (optlen) {
-        memcpy(&convtmp, &optptr[2], sizeof(uint32_t));
-        addr.s_addr = convtmp;
-        dest += sprintf(dest, "%s ", inet_ntoa(addr));
+        dest += sprintf(dest, "%s ", inet_ntoa((struct in_addr){peek(options, 4)}));
+        options += 4;
         optlen -= 4;
       }
       *(dest - 1) = '\0';
       msgopt_list[count].val = strdup(toybuf);
-      msgopt_list[count].len = strlen(toybuf);
       break;
     case DHCP_STRLST: //FIXME: do smthing.
     case DHCP_IPPLST:
@@ -1181,10 +1164,12 @@ static uint8_t dhcpc_parseoptions(dhcpc_result_t *presult, uint8_t *optptr)
         optlen -= 4;
       }
       msgopt_list[count].val = strdup(toybuf);
-      msgopt_list[count].len = strlen(toybuf);
       break;
     default: break;
     }
+    if (msgopt_list[count].val)
+      msgopt_list[count].len = strlen(msgopt_list[count].val);
+
     optptr += optptr[1] + 2;
   }
   if ((overloaded == 1) || (overloaded == 3)) dhcpc_parseoptions(presult, optptr);
@@ -1214,14 +1199,14 @@ static void renew(void)
     break;
   case STATE_BOUND:
     mode_raw();
-  case STATE_RENEWING:    // FALLTHROUGH 
-  case STATE_REBINDING:   // FALLTHROUGH 
+  case STATE_RENEWING:    // FALLTHROUGH
+  case STATE_REBINDING:   // FALLTHROUGH
     state->status = STATE_RENEW_REQUESTED;
     break;
   case STATE_RENEW_REQUESTED:
     run_script(NULL, "deconfig");
-  case STATE_REQUESTING:           // FALLTHROUGH 
-  case STATE_RELEASED:             // FALLTHROUGH 
+  case STATE_REQUESTING:           // FALLTHROUGH
+  case STATE_RELEASED:             // FALLTHROUGH
     mode_raw();
     state->status = STATE_INIT;
     break;
