@@ -4,19 +4,19 @@
  *
  * See http://opengroup.org/onlinepubs/9699919799/utilities/mkdir.html
 
-USE_MKDIR(NEWTOY(mkdir, "<1"USE_MKDIR_Z("Z:")"vpm:", TOYFLAG_BIN|TOYFLAG_UMASK))
+USE_MKDIR(NEWTOY(mkdir, "<1"USE_MKDIR_Z("Z:")"vp(parent)(parents)m:", TOYFLAG_BIN|TOYFLAG_UMASK))
 
 config MKDIR
   bool "mkdir"
   default y
   help
-    usage: mkdir [-vp] [-m mode] [dirname...]
+    usage: mkdir [-vp] [-m MODE] [DIR...]
 
     Create one or more directories.
 
-    -m	set permissions of directory to mode
-    -p	make parent directories as needed
-    -v	verbose
+    -m	Set permissions of directory to mode
+    -p	Make parent directories as needed
+    -v	Verbose
 
 config MKDIR_Z
   bool
@@ -25,7 +25,7 @@ config MKDIR_Z
   help
     usage: [-Z context]
 
-    -Z	set security context
+    -Z	Set security context
 */
 
 #define FOR_mkdir
@@ -40,7 +40,7 @@ void mkdir_main(void)
   char **s;
   mode_t mode = (0777&~toys.old_umask);
 
-  if (CFG_MKDIR_Z && (toys.optflags&FLAG_Z))
+  if (CFG_MKDIR_Z && FLAG(Z))
     if (0>lsm_set_create(TT.Z))
       perror_exit("-Z '%s' failed", TT.Z);
 
@@ -48,7 +48,7 @@ void mkdir_main(void)
 
   // Note, -p and -v flags line up with mkpathat() flags
   for (s=toys.optargs; *s; s++) {
-    if (mkpathat(AT_FDCWD, *s, mode, toys.optflags|1))
+    if (mkpathat(AT_FDCWD, *s, mode, toys.optflags|MKPATHAT_MKLAST))
       perror_msg("'%s'", *s);
   }
 }
